/*	This is a simple 'return to CD menu' library for compilation-friendly
	homebrew apps and games.
	
	It works with MeidoMenu v1.2 and any MENU.PSX boot menus.
	
*/

static int MenuType=0;

int CdMenuPresent();
void ReturnToCdMenu();

int CdMenuPresent() {

	/*
		
		This function does a simple check if a loader menu exists on the
		disc and will return a non-zero value if it does. Otherwise, it'll
		just return zero.
		
		This function is perfect if you want to know if your game was
		included in a compilation so you could enable an extra option in
		your game that'll bring the player back to the loader menu.
		
		Call this function once CdInit() has been called as it uses
		CdSearchFile() to check the existence of a CD menu.
		
	*/
	
	if (CdSearchFile(0, "\\MENU\\MENU.PSX;1")) {	// Check for a MENU.PSX menu
		MenuType=1;
		return(1);
	} else {
		if (CdSearchFile(0, "\\MMENU.EXE;1")) {		// Otherwise, look for MeidoMenu 1.2
			MenuType=2;
			return(1);
		}
		return(0);
	}
	
}

void ReturnToCdMenu() {
	
	/*
		
		This function clears the screen, sets to screen mode to 640x480, and
		finally attempts to load the CD menu.
		
		Before calling this function, you must call CdMenuPresent() first to
		identify what menu is on the disc. Also, make sure that you have stopped
		any currently playing music and/or any callbacks/threads... If possible,
		perform a fade or transition to black effect before calling this.
		
		Also, make sure that no executable code is loaded in address 0x801B2000
		and up otherwise, the menu will fail to load. Huge arrays and memory blocks
		wouldn't matter much as long as its contents are not modified while returning
		to the menu.
		
	*/
	
	typedef struct {
		u_char		pad[16];	// Useless padding
		struct EXEC	params;		// EXEC structure were going to need to start the EXE
		u_char		pad2[1972];	// More useless padding
	} EXEHEADER;
	
	DISPENV 	tdisp;
	RECT		trect={0,0,640,480};
	
	u_char		mode=CdlModeSpeed;
	EXEHEADER	exec;
	CdlFILE		file;
	
	
	if (MenuType == 0) return;
	
	// Reset the screen
	ResetGraph(0);
	
	// Prepare the new display environment and clear the screen
	SetDefDispEnv(&tdisp, 0, 0, 640, 480);
	ClearImage2(&trect, 0, 0, 0);
	DrawSync(0);
	
	// Set the new screen mode and display it
	// (so that the transition to blue effect when the menu starts will be effective)
	VSync(0);
	PutDispEnv(&tdisp);
	SetDispMask(1);	
	
	
	// Search for the file to load
	switch(MenuType) {
		case 1: // Generic boot menu
			CdSearchFile(&file, "\\MENU\\MENU.PSX;1");
			break;
		case 2: // Old Meido-Menu (that reside in the root)
			CdSearchFile(&file, "\\MMENU.EXE;1");
			break;
	}
	
	// Seek to the EXE file and read its header
	CdControl(CdlSetloc, (u_char*)&file.pos, 0);
	CdRead(1, (u_long*)&exec, mode);
	CdReadSync(0, 0);
	
	// Seek to next sector and read the rest of the EXE file
	CdIntToPos(CdPosToInt(&file.pos)+1, &file.pos);
	CdControl(CdlSetloc, (u_char*)&file.pos, 0);
	CdRead(((file.size - 2048)+2047)/2048, (void*)exec.params.t_addr, mode);
	CdReadSync(0, 0);
	
	
	// Set the stack address for the loaded EXE
	exec.params.s_addr = 0x801FFFF0;
	exec.params.s_size = 0;
	
	// Close down everything
	SpuQuit();
	ResetGraph(3);
	PadStop();
	StopCallback();
	EnterCriticalSection();
	
	// Execute the menu!
	Exec(&exec.params, 1, 0);
	
}