#ifndef true
#define true
#endif

#ifndef false
#define false
#endif

typedef struct {
	char*	FileName; 
	int		StartPos;
} XAFILE;

typedef struct {
	u_short	ID;
	u_short Chan;
	u_char	pad[28];
} XASECTOR;


static	XASECTOR	XAsector;

static	CdlLOC		XAloc;
static	CdlFILTER	XAfilter;
static	CdlCB		XAoldCallback;

static	XAFILE		*CurrentEntry;
static	char		XAvolLeft=127, XAvolRight=127;
static	int			XAcurrentChannel;
static	int			XAplayLooped;
static	int			XAplaying=0;


static int XAcallback(int intr, u_char *result);

int		XAinitFile(XAFILE *file);
void	XAsetup(int speed);
void	XAremove();
void	XAsetVolume(char volleft, char volright);
void	XAplayChannel(XAFILE *file, int channel, int looped);

static int XAcallback(int intr, u_char *result) {

	if (intr == CdlDataReady) {
	
		// Get a small nibble of the read sector
		CdGetSector((u_long*)&XAsector, 4);
		
		// Grab the first 4 bytes of the sector
		XAsector.Chan = (XAsector.Chan & 31744) >> 10;
		
		// Check to see if we've reached the end of a track by seeing if the sectors
		// being read are video sectors.
		if ((XAsector.ID == 352) && (XAsector.Chan == XAcurrentChannel)) {
			
			// We've reached the end of a track (or found a video sector), so...
			if (XAplayLooped) { // Replay XA track
				
				XAfilter.file = 1;
				XAfilter.chan = XAcurrentChannel;
				CdControlF(CdlSetfilter, (u_char*)&XAfilter);
				CdIntToPos(CurrentEntry->StartPos, &XAloc);
				CdControlF(CdlReadS, (u_char*)&XAloc);
				
				#if XADEBUG
				printf("Repeating channel %d of %s\n", XAcurrentChannel, CurrentEntry->FileName);
				#endif
				
			} else { // Stop playback altogether
				
				SsSetSerialVol(SS_SERIAL_A, 0, 0);
				CdControlF(CdlPause, 0);
				
				#if XADEBUG
				printf("XA playback finished.\n", XAcurrentChannel, CurrentEntry->FileName);
				#endif
				
			}
			
		}
		
	}
	
}
int XAinitFile(XAFILE *file) {
	
	/* Returns:
		0 - Init ok
		1 - Failed
	*/
	
	CdlFILE	fp;
	
	#if XADEBUG
	printf("Searching XA file %s...", file->FileName);
	#endif
	
	if (CdSearchFile(&fp, file->FileName) == 0) {
		#if XADEBUG
		printf("ERROR: XA file not found!\n");
		#endif
		return(1);
	}
	
	#if XADEBUG
	printf("Ok!\n");
	#endif
	
	file->StartPos = CdPosToInt(&fp.pos);
	
	return(0);
	
}
void XAsetup(int speed) {
	
	u_char param[4]={0};
	
	#if XADEBUG
	printf("Setting up XA streaming mode...");
	#endif
	
	// Ready the XA system for playing
	if (speed == 2) {
		param[0] = CdlModeSpeed|CdlModeRT|CdlModeSF;	// 2x speed
	} else {
		param[0] = CdlModeRT|CdlModeSF; 				// 1x speed
	}
	
	CdControlB(CdlSetmode, param, 0);
	CdControlF(CdlPause, 0);
	CdSync(0, 0);
	
	XAoldCallback = CdReadyCallback((CdlCB)XAcallback);
	
	#if XADEBUG
	printf("Done!\n");
	#endif
	
}
void XAremove() {
	
	u_char param[4];
	
	#if XADEBUG
	printf("Removing XA streaming mode...");
	#endif
	
	// Mute so that no garbage will be played
	SsSetSerialVol(SS_SERIAL_A, 0, 0);
	
    // reset any callback that we replaced
	CdControlF(CdlPause,0);
	CdReadyCallback((void*)XAoldCallback);

    // clear XA mode
	param[0] = CdlModeSpeed;
	CdControlB(CdlSetmode, param, 0);
	
	#if XADEBUG
	printf("Done!\n");
	#endif
	
}
void XAsetVolume(char volleft, char volright) {
	
	XAvolLeft	= volleft;
	XAvolRight	= volright;
	
	if (XAplaying == false) return;
	
	SsSetSerialVol(SS_SERIAL_A, volleft, volright);
	
}
void XAplayChannel(XAFILE *file, int channel, int looped) {
	
	#if XADEBUG
	printf("Playing channel %d of %s...\n", channel, file->FileName);
	#endif
	
	// Mute and pause so we won't hear the other channels while changing
	SsSetSerialVol(SS_SERIAL_A, 0, 0);
	CdControlF(CdlPause, 0);
	
	// set up the XA filter
	XAfilter.file = 1;
	XAfilter.chan = channel;
	CdControlF(CdlSetfilter, (u_char*)&XAfilter);
	
	// Set starting position on the CD
	CdIntToPos(file->StartPos, &XAloc);

	// Begin playing
	CdControlF(CdlReadS, (u_char*)&XAloc);
	
	// Set volume to max
	SsSetSerialVol(SS_SERIAL_A, XAvolLeft, XAvolRight);
	
	CurrentEntry		= file;
	XAcurrentChannel	= channel;
	XAplayLooped		= looped;
	XAplaying			= 1;
	
}